/*
 * mp3plot - Bitrate analysis tool
 *
 * Copyright (C) 2007, 2009 Toni Corvera
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

// $Id: main.cc 1652 2009-04-14 23:57:40Z  $

#include <fstream>
#include <string>
#include <iostream>
#include <ostream>

#include <cassert>
#include <cstring>

#include <boost/program_options.hpp>
#include <boost/version.hpp>

#include "mp3_config.h"
#include "mp3_file.h"
#include "main.h"
#include "plotters/init_plotters.h"
#include "plotters/bitrate_plotter.h"
#include "utils/str_utils.h"
#include "utils/logger.h"

#define E_HEADER_READ   70
#define E_BAD_HEADER    71

namespace mp3 = mp3plot;
namespace po  = boost::program_options;
namespace str = net_outlyer::str;

namespace { // anon ns
    using namespace std;

    inline bool file_exists(const std::string & filename) {
        std::ifstream ifs(filename.c_str());
        ifs.close();
        return ! ifs.fail();
    }

    inline void print_strvector(ostream& os, const vector<string> & v) {
        for (vector<string>::const_iterator it = v.begin(); it != v.end(); ++it) {
            os << "* " << (*it) << "\n";
        }
    }

    void print_signature(std::ostream &);
    void print_usage(std::ostream &);
    void print_versions(std::ostream &);

    std::string progname;
} // anon ns

#if 0 // moved to <anon>::print_strvector
std::ostream & operator<<(std::ostream & os, const std::vector<std::string> & v) {
    typedef std::vector<std::string> strvec;

    for (strvec::const_iterator it = v.begin(); it != v.end(); ++it) {
        os << "* " << *it << "\n";
    }

    return os;
}
#endif

int main(int argc, char * argv[], char * envp[]) {
    progname = argv[0];

    mp3::register_plotters(argv[0]);

    using namespace std;
    using net_outlyer::logger;

    string inputfile, outputfile, plotter_by_name;
    bool image_output;

    po::options_description desc("Available options");
    desc.add_options()
        ("input,i", po::value<string>(&inputfile), "input file")
        ("output,o", po::value<string>(&outputfile), "output file (only needed for image output)")
        ("image,g", "output to an image"
#if !defined(HAVE_GRAPHICS_SUPPORT) || defined(DISABLE_GRAPHICS)
         " (disabled)"
#endif
         )
        ("named-plotter,p", po::value<string>(&plotter_by_name),
                                "manually choose plotter (use \"list\" for "
                                "a list of available plotters)")
        ("help,h", "show this text")
        ("version,V", "show version information")
    ;

    // all positional = unnamed args...
    po::positional_options_description pos;
    pos.add("input", -1); // ... are equivalent to --input <arg>

    po::variables_map vm;
    try {
        po::store(po::command_line_parser(argc, argv)
                                        .options(desc)
                                        .positional(pos).run(), vm);
    }
    catch (const po::unknown_option & e) {
        std::cerr << desc;
        std::cerr << "\nERROR: " << e.what();
        exit(EX_USAGE);
    }
    catch (const po::invalid_command_line_syntax & e) {
        std::cerr << desc;
        std::cerr << "\nERROR: " << e.what();
        exit(EX_USAGE);
    }
    catch (const po::multiple_occurrences & e) {
        std::cerr << desc;
        std::cerr << "\nERROR: " << e.what();
        exit(EX_USAGE);
    }
    po::notify(vm);

    image_output = (0 != vm.count("image"));

    if (0 != vm.count("help")) {
        print_signature(cout);
        print_usage(cout);
        cout << desc << endl;
        exit(EX_OK);
    }

    if (0 != vm.count("version")) {
        print_signature(cout);
        print_versions(cout);
        exit(EX_OK);
    }

    // Make it case-insensitive
    plotter_by_name = str::to_lower(plotter_by_name);
    // "list" is a special plotter which will show the available plotters
    // and exit immediately
    if (plotter_by_name == "list") {
        cerr << "Avilable plotters:\n";
        print_strvector(cerr, mp3::bitrate_plotter::list());
        cerr << endl;
        exit(0);
    }

    if (0 == vm.count("input")) { // FIXME!: Really Support more than one input
        print_signature(cerr);
        cerr << "No input file provided" << endl;
        print_usage(cerr);
        cerr << desc;
        exit(EX_USAGE);
    }

#if 0
    if (!vm.count("named-plotter")) {
    }
#endif

    print_signature(cout);

    if (!file_exists(inputfile)) {
        cerr << "Failed to read from input file " << inputfile << endl;
        return EX_NOINPUT;

    }

    try {
        using mp3::bitrate_plotter;
        using mp3::plotter_features;
        using mp3::mp3file;

        bitrate_plotter * pbrp = 0;
        plotter_features feats;

        // Choose the plotter before continuing (no need to parse the file
        // on error)
        if (plotter_by_name.length() > 0) {
            // A plotter name was specified, try to choose it...
            pbrp =& bitrate_plotter::choose(plotter_by_name);
        }
        else {
            // No plotter manually choosed, decide based on type...
            if (!image_output) {
                feats = plotter_features::textual() | plotter_features::printed();
            }
            else {
                feats = plotter_features::graphical() | plotter_features::file();
            }

            pbrp =& bitrate_plotter::choose_fuzzy(feats);
        }
        // POST: a bitrate plotter was chosen or an exception thrown
        assert( 0 != pbrp );

        if (outputfile.empty()) {
            outputfile = inputfile+".png";
        }
        const mp3::plot_params params(outputfile);

        mp3file file(inputfile);

        file.read_info();

        std::cout << file;
        if (image_output && !str::ends_in(outputfile, ".png")) {
            logger::os() << "Warning: output filename doesn't end in '.png', "
                            "but output *will* be PNG\n";
        }
        if (!image_output) {
            std::cout << "* Bitrate plot:\n";
        }

        file.plot(*pbrp, params); // might throw
    }
    catch (mp3::e_error & e) {
        std::cerr << progname << ": " << e.what() << std::endl;
        exit(e.error_code);
    }

    return 0;
}

// shorthands, local to main.cc, not to be used anywhere else
// XXX: This stuff might be moved to the appropriate plotters for
//      better separation
#if defined(HAVE_GD) && !defined(DISABLE_GRAPHICS) && !defined(DISABLE_GD)
#   define GDLIB_AVAILABLE
#   include <gd.h>
#endif
#if defined(HAVE_MAGICKPP) && !defined(DISABLE_GRAPHICS) && !defined(DISABLE_MAGICK)
#   define MAGICKPP_AVAILABLE
#   include <magick/version.h>
#endif

#if defined(__GNUC__) || defined(_MSC_VER)
#   define HAVE_COMPILER_INFO
#   ifdef __GNUC__
#       define COMPILER_NAME  "g++"
#       define COMPILER_VER_IOSTREAM __VERSION__
#   elif defined(_MSC_VER)
#       define COMPILER_NAME  "Microsoft's Visual C++"
#       define COMPILER_VER_IOSTREAM (_MSC_VER / 100)<<"."<<(_MSC_VER % 100)
#   endif
#endif

namespace {
    using namespace std;

    inline void print_signature(ostream & os) {
        os << PACKAGE " " VERSION " (c) " COPYRIGHT_YEAR " " AUTHOR " <" UPSTREAM_URL ">\n\n";
    }

    inline void print_usage(ostream & os) {
        os << "Usage: mp3plot <file.mp3>" << endl;
    }

    inline void print_versions(ostream& os) {
        os << "Boost version: " << (BOOST_VERSION / 100000) << "." <<
                                   (BOOST_VERSION / 100 % 1000) << "." <<
                                   (BOOST_VERSION % 100) << "\n";
#ifdef GDLIB_AVAILABLE
        os << "GD lib version: " GD_VERSION_STRING "\n";
#endif
#ifdef MAGICKPP_AVAILABLE
        os << "Magick++ lib variant and version: " MagickPackageName "++ v" MagickLibVersionText "\n";
#endif
// ANSI standard macro, but apparently G++-4.1 doesn't define it
#ifdef __TIMESTAMP__
        os << "Date of compilation: " __TIMESTAMP__ << "\n";
#endif
#ifdef HAVE_COMPILER_INFO
        os << "Compiled with " COMPILER_NAME " " << COMPILER_VER_IOSTREAM << "\n";
#endif
        os << flush;
    }
}

// vim:set ts=4 et ai:
