/*
 * mp3plot - Bitrate analysis tool
 *
 * Copyright (C) 2007, 2009 Toni Corvera
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

// $Id: image_plotter.cc 1601 2009-04-09 01:03:51Z  $

#include "image_plotter.h"

#ifndef DISABLE_GRAPHICS

#include <iostream>
#include "../mp3/bitrate.h"

namespace mp3plot {

const unsigned int image_plotter::IMGW = 1024,
                   image_plotter::IMGH = 768;

/*!
 * \brief Returns a component based on the bitrate.
 * The bitrate must match one of lower, low, med, high or higher,
 * or it will fail.
 * Each of the possible values has an intensity associated, e.g.
 * <tt>br == lower</tt> will have a lower intensity than
 * <tt>br == higher</tt>.
 * \param[in] br Bitrate
 * \todo TODO: Throw on failure
 */
inline double cfromrange(bitrate_t br,
                         bitrate_t lower,
                         bitrate_t low,
                         bitrate_t med,
                         bitrate_t high,
                         bitrate_t higher)
{
    double rv;

    if (br == lower) {
        rv = 0.3L;
    }
    else if (br == low) {
        rv = 0.4L;
    }
    else if (br == med) {
        rv = 0.5L;
    }
    else if (br == high) {
        rv = 0.6L;
    }
    else if (br == higher) {
        rv = 0.7L;
    }
    else {
        rv = .0L;
        std::cerr << br << std::endl;
        assert( false );
    };

    return rv;
}

/*!
 * \remarks The range defines which component will be variable (the other
 * components will be zero), and the variable component is obtained
 * by calling cfromrange(), so it will be more intense the higher the
 * bitrate.
 */
rgb_type image_plotter::bitrate_colour(bitrate_t br) {
    // TODO: Abstract

    // Ranges:
    static const bitrate_t low = 56, mid = 128; // , high = 320;

    double r, g, b;

    if (br == standard_bitrates::bad_bitrate()) {
        // #f3eea5
        r = 0xF3U / 255.0F;
        g = 0xEEU / 255.0F;
        b = 0xA5U / 255.0F;
    }
    else if (br <= low) {
        b = cfromrange(br, 0, 32, 40, 48, 56);
        r = .0L, g = .0L;
    }
    else if (br <= mid) {
        g = cfromrange(br, 64, 80, 96, 112, 128);
        r = .0L, b = .0L;
    }
    else {
        r = cfromrange(br, 160, 192, 224, 256, 320);
        g = .0L, b = .0L;
    }


    return rgb_type(r, g, b);
}

} // namespace mp3plot

#endif // DISABLE_GRAPHICS

// vim:set ts=4 et ai: //
